import types

import zope.interface

from collective.batch import interfaces

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'epytext'
__copyright__ = "2009 Twinapex Research"
__licence__ = "GPL v2.1"

class Batch(object):
    """ Base batch implementation. See IBatch.

    This class can be used when there exists iterable data which we cannot access normally.
    E.g. when the search result batch is returned by external service and we cannot posssible
    know whether there exists data beyond our current result set.
    """

    zope.interface.implements(interfaces.IBatch)

    def __init__(self, start, count, batchSize, totalCount, items):
        """
        See IExternalIndexResultBatch
        """

        assert batchSize != 0

        self.start = start
        self.count = count
        self.batchSize = batchSize
        self.totalCount = totalCount
        self.items = items

    def __len__(self):
        """ Spoof iterable methods for plone.app.content.batching.Batch

        @return: Total count of search results
        """
        return self.totalCount

    def __getitem__(self, key):
        """ Batch start index adjusted accessor.

        Access items so that one can assume index zero
        is the first batch result on the first batch page.

        See http://docs.python.org/reference/datamodel.html
        """

        assert type(key) == types.IntType, "Bad key:" + str(key)

        if key < self.start or key >= self.start + self.count:
            raise ValueError("Querying index %d which is not in this batch. Start index %d, item count %d" % (key, self.start, self.count))

        key = key - self.start
        return self.items[key]

    def hasSeveralBatches(self):
        """ @return True: The result set has more than one batch with the current batch size settings """
        return (self.batchSize < self.totalCount) and (self.totalCount != 0)

    def getBatchIndex(self):
        """
        @return: Number of this batch slot. Zero based.
        """

        return int(self.start / self.batchSize)

    def getPageNumber(self):
        """
        @return: Number of this page. One based.
        """
        return self.getBatchIndex() + 1

    def getItems(self):
        """
        @return: Result items as Python list.
        """
        return self.items

    @staticmethod
    def makeBatch(limit, results):
        """ Chop Python list to a batch subset.

        @param limit: tuple of (start index, page size)
        @param results: Iterable result data.
        """
        clippedResults = results[limit[0]:limit[0]+limit[1]]
        count = len(clippedResults)
        return Batch(limit[0], count, limit[1], len(results), clippedResults)
